import { message } from 'antd';
import { routerRedux } from 'dva/router';
import { Hotax } from '../utils/config';
import { query as queryUsers } from '../services/user';
import { getLocalUser } from '../utils/authority';
import {
  queryUserTags,
  createConfigUserTagItem,
  queryTagDetail,
  updateConfigUserTagItem,
  deleteConfigUserTagItem,
  queryConfigCourse,
  updateConfigCourse,
  copyTag,
} from '../services/configUser';


export default {
  namespace: 'configUser',
  state: {
    list: [],
    currentUser: {},
    TagList: [],
    userTagLists: [],
    TagDetails: [],
    courseList: [],
  },
  effects: {
    *fetch(_, { call, put }) {
      const response = yield call(queryUsers);
      yield put({
        type: 'save',
        payload: response,
      });
    },
    *fetchCurrent(_, { put }) {
      // TODO 这里从本地读取用户信息,后期改成请求
      const userInfo = getLocalUser();
      yield put({
        type: 'saveCurrentUser',
        payload: userInfo,
      });
    },
    // 用户终端下用户推荐课程相关;
    *fetchConfigCourse({ payload }, { call, put }) {
      const response = yield call(queryConfigCourse, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            courseList: response.data || [],
          },
        });
      }
    },
    *updateConfigCourse({ payload, states }, { call, put }) {
      const response = yield call(updateConfigCourse, payload);
      if (response.success) {
        message.success('推荐课程列表更新成功');
        yield put(routerRedux.push({
          pathname: '/frontend/ConfigUser',
          state: states,
        }));
      }
    },
    // 用户标签的相关
    *fetchTagDetail({ payload }, { call, put }) {
      const response = yield call(queryTagDetail, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            TagDetails: response.data || [],
          },
        });
      }
    },
    *fetchUserTags({ payload }, { call, put }) {
      const response = yield call(queryUserTags, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            userTagLists: response.data || [],
          },
        });
      } else {
        yield put({
          type: 'querySuccess',
          payload: {
            userTagLists: [],
          },
        });
      }
    },
    *createConfigUserTagItem({ payload, TagId }, { call, put, select }) {
      const response = yield call(createConfigUserTagItem, payload);
      if (response.success) {
        message.success('创建终端标签成功');
        const originalData = yield select(state => state.configUser.userTagLists);
        const newData = originalData.map((data) => {
          const { id, isNew, isEdit, ...rest } = data;
          if (id === TagId) {
            return { ...rest, id: response.data.id };
          }
          return { ...data };
        });
        yield put({
          type: 'fixUserTagLists',
          payload: newData,
        });
      }
    },
    *updateConfigUserTagItem({ payload, TagId }, { call, put, select }) {
      const response = yield call(updateConfigUserTagItem, payload);
      if (response.success) {
        message.success('修改终端标签成功');
        const originalData = yield select(state => state.configUser.userTagLists);
        const newData = originalData.map((data) => {
          const { isEdit, ...rest } = data;
          if (data.id === TagId) {
            return { ...rest };
          }
          return { ...data };
        });
        yield put({
          type: 'fixUserTagLists',
          payload: newData,
        });
      }
    },
    *deleteConfigUserTagItem({ payload }, { call, put, select }) {
      const response = yield call(deleteConfigUserTagItem, payload);
      if (response.success) {
        message.success('删除终端标签成功');
        const originalData = yield select(state => state.configUser.userTagLists);
        const newData = originalData.map((data) => {
          if (data.id === payload) {
            return { ...data, status: Hotax.STATUS_DELETE };
          }
          return { ...data };
        });
        yield put({
          type: 'fixUserTagLists',
          payload: newData,
        });
      }
    },
    *copyTag({ payload }, { call }) {
      const response = yield call(copyTag, payload);
      if (response.success) {
        message.success('成功复制标签关联产品到当前标签');
      }
    },
  },
  reducers: {
    querySuccess(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    fixCourseList(state, action) {
      return {
        ...state,
        courseList: action.payload,
      };
    },
    save(state, action) {
      return {
        ...state,
        list: action.payload,
      };
    },
    fixUserTagLists(state, action) {
      return {
        ...state,
        userTagLists: action.payload,
      };
    },
    saveCurrentUser(state, action) {
      return {
        ...state,
        currentUser: action.payload,
      };
    },
    changeNotifyCount(state, action) {
      return {
        ...state,
        currentUser: {
          ...state.currentUser,
          notifyCount: action.payload,
        },
      };
    },
  },
};
