import React, { Component } from 'react';
import pathToRegexp from 'path-to-regexp';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Button } from 'antd';
import AXDragSortTable from '../../../components/AXDragSortTable';
import Selector from '../../../components/AXTableSelector/Selector';
import FooterToolbar from '../../../components/FooterToolbar/index';

@connect(({ loading, merchant, shelves, configUser }) => ({
  shelves,
  merchant,
  configUser,
  sLoading: loading.models.product,
  submitting: loading.models.merchant,
}))

export default class ConfigCourse extends Component {
  state = {
    productSelectorDestroy: true,
  };
  componentDidMount() {
    this.props.dispatch({
      type: 'configUser/fetchConfigCourse',
      payload: { courseId: this.getMerchantId() },
    });
  }
  getMerchantId = () => {
    const match = pathToRegexp('/frontend/ConfigUser/course-edit/:id')
      .exec(this.props.location.pathname);
    return match[1];
  }
  handleSelectorModalShow = () => {
    const { itemData } = this.props.location.state;
    const { merchantId: id } = itemData;
    this.setState({ productSelectorDestroy: false });
    this.props.dispatch({
      type: 'shelves/fetchCourseItemList',
      payload: { merchantId: id },
    });
  }
  handleSelectorChange = (params) => {
    const { itemData } = this.props.location.state;
    const { merchantId: id } = itemData;
    this.props.dispatch({
      type: 'shelves/fetchCourseItemList',
      payload: {
        merchantId: id,
        ...params,
      },
    });
  }
  handleSelectorFinish = (rows) => {
    this.setState({ productSelectorDestroy: true });
    this.props.dispatch({
      type: 'configUser/fixCourseList',
      payload: rows,
    });
  }
  handleSelectorCancel = () => {
    this.setState({ productSelectorDestroy: true });
  }
  handleDragSortTableChange = (rows) => {
    this.props.dispatch({
      type: 'configUser/fixCourseList',
      payload: rows,
    });
  }
  handlePageBack = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/frontend/ConfigUser',
      state: this.props.location.state,
    }));
  }
  handlePageSubmit = () => {
    const { configUser } = this.props;
    const { courseList } = configUser;
    const idList = courseList.map(item => item.pid);
    this.props.dispatch({
      type: 'configUser/updateConfigCourse',
      payload: {
        idList,
        courseId: this.getMerchantId(),
      },
    });
  }

  render() {
    const { productSelectorDestroy } = this.state;
    const { shelves, configUser } = this.props;
    const { courseList } = configUser;
    // recommend事件
    const productColumns = [{
      title: '课程编号',
      key: 1,
      dataIndex: 'code',
      width: '40%',
    }, {
      title: '课程名称',
      key: 2,
      dataIndex: 'name',
    }];
    const getProductModal = () => {
      return (
        <Modal
          visible
          width={1100}
          footer={null}
          title="课程资源"
          maskClosable={false}
          onCancel={this.handleSelectorCancel}
        >
          <Selector
            multiple
            selectorName="Course"
            selectedRows={courseList}
            list={shelves.list}
            pageNo={shelves.pageNo}
            pageSize={shelves.pageSize}
            totalSize={shelves.totalSize}
            onCancel={this.handleSelectorCancel}
            onChange={this.handleSelectorChange}
            onFinish={this.handleSelectorFinish}
          />
        </Modal>
      );
    };
    return (
      <div>
        <Card
          title={<a onClick={this.handleSelectorModalShow}>选择课程</a>}
          style={{ marginBottom: 70 }}
        >
          <AXDragSortTable
            columns={productColumns}
            data={courseList}
            onChange={this.handleDragSortTableChange}
          />
          {!productSelectorDestroy && getProductModal()}
        </Card>
        <FooterToolbar style={{ width: '100%' }}>
          <Button
            onClick={this.handlePageBack}
            style={{ marginRight: 10 }}
          >取消
          </Button>
          <Button
            type="primary"
            onClick={this.handlePageSubmit}
          >提交
          </Button>
        </FooterToolbar>
      </div>
    );
  }
}
