import React, { Component } from 'react';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Form, Menu, Dropdown, Icon } from 'antd';
import { StandardTableList } from '../../../components/AXList/index';
import Ellipsis from '../../../components/Ellipsis/index';
import { addRowKey, renderStatus } from '../../../utils/utils';

@Form.create()
@connect(({ loading, campus, merchant, terminal }) => ({
  campus,
  merchant,
  terminal,
  fetching1: loading.models.merchant,
  fetching2: loading.models.campus,
  loading: loading.models.terminal,
}))

export default class ConfigUserLists extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
      filterModalDestroy: true,
      itemData: {},
    };
  }
  componentDidMount() {
    this.props.dispatch({
      type: 'terminal/fetchTerminalList',
      payload: { ...this.state.Queryers },
    });
  }
  // 跳转到配置标签的page
  handleEditOperation = (item) => {
    this.props.dispatch(routerRedux.push({
      pathname: `/frontend/ConfigUser/tag/${item.id}`,
      state: {
        currentItem: item,
        ...this.state,
      },
    }));
  }
  // 跳转到配置课程页面
  handleDeviceCourseOperation = (item) => {
    const { id } = item;
    this.props.dispatch(routerRedux.push({
      pathname: `/frontend/ConfigUser/course-edit/${id}`,
      state: this.state,
    }));
  }
  // 跳到配置海报的页面
  handleDevicePosterOperation = (item) => {
    const { id } = item;
    this.props.dispatch(routerRedux.push({
      pathname: `/frontend/ConfigUser/poster-edit/${id}`,
      state: this.state,
    }));
  }
  handleFilterOperation = (params, states) => {
    this.props.dispatch({
      type: 'terminal/fetchTerminalList',
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  }
  // 将item中的数据传送给子栏目
  transferData = (item) => {
    this.setState(() => {
      return {
        itemData: item,
      };
    });
  }
  render() {
    const { loading, terminal } = this.props;
    const { list, totalSize, pageSize, pageNo } = terminal;
    const { itemData } = this.state;

    const renderCampusName = (name) => {
      return (
        <Ellipsis tooltip lines={1}>{name}</Ellipsis>
      );
    };
    const menu = (
      <Menu>
        <Menu.Item>
          <a onClick={() => this.handleDevicePosterOperation(itemData)}>
            推荐海报
          </a>
        </Menu.Item>
        <Menu.Item>
          <a onClick={() => this.handleDeviceCourseOperation(itemData)}>
            推荐课程
          </a>
        </Menu.Item>
      </Menu>
    );
    const renderOperation = (item) => {
      return (
        <div>
          <a
            onClick={() => this.handleEditOperation(item)}
          >标签配置
          </a>
          &nbsp;&nbsp;&nbsp;
          <Dropdown overlay={menu} trigger={['click']}>
            <a
              onClick={() => this.transferData(item)}
            >推荐配置 <Icon type="down" />
            </a>
          </Dropdown>
        </div>
      );
    };
    const basicSearch = {
      keys: [{
        name: '终端编号',
        field: 'code',
      }, {
        name: '终端名称',
        field: 'name',
      }],
    };
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: '终端编号',
      key: 1,
      dataIndex: 'code',
      width: '20%',
    }, {
      title: '终端名称',
      key: 2,
      dataIndex: 'name',
      width: '20%',
    }, {
      title: '所属校区',
      key: 3,
      dataIndex: 'campusName',
      render: text => renderCampusName(text),
      width: '22%',
    }, {
      title: '所属渠道',
      key: 4,
      dataIndex: 'merchantName',
      width: '12%',
    }, {
      title: '账号状态',
      key: 5,
      dataIndex: 'status',
      render: text => renderStatus(text, '已禁用'),
      width: '8%',
    }, {
      title: '操作',
      key: 6,
      dataIndex: 'operation',
      render: (_, record) => renderOperation(record),
      width: '20%',
      align: 'right',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onFilterClick: this.handleFilterOperation,
          }}
          footer={{
            pagination,
          }}
          keepUIState={{ ...this.state.UIParams }}
        />
      </Card>
    );
  }
}
